@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -31,6 +31,8 @@
 
 from io import IOBase
 import os
+import platform
+import socket
 import time
 
 from .authentication import get_auth_plugin
@@ -40,7 +42,7 @@ from .constants import (
     flag_is_set, ShutdownType, NET_BUFFER_LENGTH
 )
 
-from . import errors
+from . import errors, version
 from .conversion import MySQLConverter
 from .cursor import (
     CursorBase, MySQLCursor, MySQLCursorRaw,
@@ -49,7 +51,7 @@ from .cursor import (
     MySQLCursorBufferedNamedTuple)
 from .network import MySQLUnixSocket, MySQLTCPSocket
 from .protocol import MySQLProtocol
-from .utils import int4store
+from .utils import int4store, linux_distribution
 from .abstracts import MySQLConnectionAbstract
 
 
@@ -106,6 +108,39 @@ class MySQLConnection(MySQLConnectionAbstract):
                 self._socket = None
                 raise
 
+    def _add_default_conn_attrs(self):
+        """Add the default connection attributes."""
+        if os.name == "nt":
+            if "64" in platform.architecture()[0]:
+                platform_arch = "x86_64"
+            elif "32" in platform.architecture()[0]:
+                platform_arch = "i386"
+            else:
+                platform_arch = platform.architecture()
+            os_ver = "Windows-{}".format(platform.win32_ver()[1])
+        else:
+            platform_arch = platform.machine()
+            if platform.system() == "Darwin":
+                os_ver = "{}-{}".format("macOS", platform.mac_ver()[0])
+            else:
+                os_ver = "-".join(linux_distribution()[0:2])
+
+        license_chunks = version.LICENSE.split(" ")
+        if license_chunks[0] == "GPLv2":
+            client_license = "GPL-2.0"
+        else:
+            client_license = "Commercial"
+        default_conn_attrs = {
+            "_pid": str(os.getpid()),
+            "_platform": platform_arch,
+            "_source_host": socket.gethostname(),
+            "_client_name": "mysql-connector-python",
+            "_client_license": client_license,
+            "_client_version": ".".join([str(x) for x in version.VERSION[0:3]]),
+            "_os": os_ver}
+
+        self._conn_attrs.update((default_conn_attrs))
+
     def _do_handshake(self):
         """Get the handshake from the MySQL server"""
         packet = self._socket.recv()
@@ -116,8 +151,9 @@ class MySQLConnection(MySQLConnectionAbstract):
         try:
             handshake = self._protocol.parse_handshake(packet)
         except Exception as err:
-            raise errors.InterfaceError(
-                'Failed parsing handshake; {0}'.format(err))
+            # pylint: disable=E1101
+            raise errors.get_mysql_exception(msg=err.msg, errno=err.errno,
+                                             sqlstate=err.sqlstate)
 
         self._server_version = self._check_server_version(
             handshake['server_version_original'])
@@ -136,7 +172,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._handshake = handshake
 
     def _do_auth(self, username=None, password=None, database=None,
-                 client_flags=0, charset=45, ssl_options=None):
+                 client_flags=0, charset=45, ssl_options=None, conn_attrs=None):
         """Authenticate with the MySQL server
 
         Authentication happens in two parts. We first send a response to the
@@ -151,12 +187,18 @@ class MySQLConnection(MySQLConnectionAbstract):
             packet = self._protocol.make_auth_ssl(charset=charset,
                                                   client_flags=client_flags)
             self._socket.send(packet)
+            if ssl_options.get('tls_ciphersuites') is not None:
+                tls_ciphersuites = ":".join(ssl_options.get('tls_ciphersuites'))
+            else:
+                tls_ciphersuites = ""
             self._socket.switch_to_ssl(ssl_options.get('ca'),
                                        ssl_options.get('cert'),
                                        ssl_options.get('key'),
                                        ssl_options.get('verify_cert') or False,
-                                       ssl_options.get('cipher'),
-                                       ssl_options.get('version', None))
+                                       ssl_options.get('verify_identity') or
+                                       False,
+                                       tls_ciphersuites,
+                                       ssl_options.get('tls_versions'))
             self._ssl_active = True
 
         packet = self._protocol.make_auth(
@@ -164,7 +206,8 @@ class MySQLConnection(MySQLConnectionAbstract):
             username=username, password=password, database=database,
             charset=charset, client_flags=client_flags,
             ssl_enabled=self._ssl_active,
-            auth_plugin=self._auth_plugin)
+            auth_plugin=self._auth_plugin,
+            conn_attrs=conn_attrs)
         self._socket.send(packet)
         self._auth_switch_request(username, password)
 
@@ -246,7 +289,7 @@ class MySQLConnection(MySQLConnectionAbstract):
             self._do_handshake()
             self._do_auth(self._user, self._password,
                           self._database, self._client_flags, self._charset_id,
-                          self._ssl)
+                          self._ssl, self._conn_attrs)
             self.set_converter_class(self._converter_class)
             if self._client_flags & ClientFlag.COMPRESS:
                 self._socket.recv = self._socket.recv_compressed
@@ -396,6 +439,8 @@ class MySQLConnection(MySQLConnectionAbstract):
         """Handle a LOAD DATA INFILE LOCAL request"""
         try:
             data_file = open(filename, 'rb')
+            return self._handle_ok(self._send_data(data_file,
+                                                   send_empty_packet=True))
         except IOError:
             # Send a empty packet to cancel the operation
             try:
@@ -405,9 +450,11 @@ class MySQLConnection(MySQLConnectionAbstract):
                     "MySQL Connection not available.")
             raise errors.InterfaceError(
                 "File '{0}' could not be read".format(filename))
-
-        return self._handle_ok(self._send_data(data_file,
-                                               send_empty_packet=True))
+        finally:
+            try:
+                data_file.close()
+            except (IOError, NameError):
+                pass
 
     def _handle_result(self, packet):
         """Handle a MySQL Result
@@ -468,7 +515,8 @@ class MySQLConnection(MySQLConnectionAbstract):
             return (rows[0], eof)
         return (None, eof)
 
-    def get_rows(self, count=None, binary=False, columns=None, raw=None):
+    def get_rows(self, count=None, binary=False, columns=None, raw=None,
+                 prep_stmt=None):
         """Get all rows returned by the MySQL server
 
         This method gets all rows returned by the MySQL server after sending,
