@@ -1,4 +1,4 @@
-# Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -177,7 +177,8 @@ class Schema(DatabaseObject):
         Returns:
             `list`: List of Collection objects.
         """
-        rows = self._connection.get_row_result("list_objects", self._name)
+        rows = self._connection.get_row_result("list_objects",
+                                               {"schema": self._name})
         rows.fetch_all()
         collections = []
         for row in rows:
@@ -205,7 +206,8 @@ class Schema(DatabaseObject):
         Returns:
             `list`: List of Table objects.
         """
-        rows = self._connection.get_row_result("list_objects", self._name)
+        rows = self._connection.get_row_result("list_objects",
+                                               {"schema": self._name})
         rows.fetch_all()
         tables = []
         object_types = ("TABLE", "VIEW",)
@@ -283,8 +285,9 @@ class Schema(DatabaseObject):
             raise ProgrammingError("Collection name is invalid")
         collection = Collection(self, name)
         if not collection.exists_in_database():
-            self._connection.execute_nonquery("xplugin", "create_collection",
-                                              True, self._name, name)
+            self._connection.execute_nonquery("mysqlx", "create_collection",
+                                              True, {"schema": self._name,
+                                                     "name": name})
         elif not reuse:
             raise ProgrammingError("Collection already exists")
         return collection
@@ -315,7 +318,9 @@ class Collection(DatabaseObject):
             condition (Optional[str]): The string with the filter expression of
                                        the documents to be retrieved.
         """
-        return FindStatement(self, condition)
+        stmt = FindStatement(self, condition)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def add(self, *values):
         """Adds a list of documents to a collection.
@@ -342,7 +347,9 @@ class Collection(DatabaseObject):
         .. versionchanged:: 8.0.12
            The ``condition`` parameter is now mandatory.
         """
-        return RemoveStatement(self, condition)
+        stmt = RemoveStatement(self, condition)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def modify(self, condition):
         """Modifies documents based on the ``condition``.
@@ -357,7 +364,9 @@ class Collection(DatabaseObject):
         .. versionchanged:: 8.0.12
            The ``condition`` parameter is now mandatory.
         """
-        return ModifyStatement(self, condition)
+        stmt = ModifyStatement(self, condition)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def count(self):
         """Counts the documents in the collection.
@@ -389,6 +398,7 @@ class Collection(DatabaseObject):
                                    {"fields": [{"field": member_path,
                                                 "type": member_type,
                                                 "required": member_required,
+                                                "array": array,
                                                 "collation": collation,
                                                 "options": options,
                                                 "srid": srid},
@@ -406,9 +416,10 @@ class Collection(DatabaseObject):
         Args:
             index_name (str): Index name.
         """
-        self._connection.execute_nonquery("xplugin", "drop_collection_index",
-                                          False, self._schema.name, self._name,
-                                          index_name)
+        self._connection.execute_nonquery("mysqlx", "drop_collection_index",
+                                          False, {"schema": self._schema.name,
+                                                  "collection": self._name,
+                                                  "name": index_name})
 
     def replace_one(self, doc_id, doc):
         """Replaces the Document matching the document ID with a new document
@@ -416,7 +427,7 @@ class Collection(DatabaseObject):
 
         Args:
             doc_id (str): Document ID
-            doc (DbDoc/dict): New Document
+            doc (:class:`mysqlx.DbDoc` or `dict`): New Document
         """
         return self.modify("_id = :id").set("$", doc) \
                    .bind("id", doc_id).execute()
@@ -427,7 +438,7 @@ class Collection(DatabaseObject):
 
         Args:
             doc_id (str): Document ID
-            doc (DbDoc/dict): New Document
+            doc (:class:`mysqlx.DbDoc` or dict): New Document
         """
         if not isinstance(doc, DbDoc):
             doc = DbDoc(doc)
@@ -438,8 +449,14 @@ class Collection(DatabaseObject):
 
         Args:
             doc_id (str): Document ID
+
+        Returns:
+            mysqlx.DbDoc: The Document matching the Document ID.
         """
-        return self.find("_id = :id").bind("id", doc_id).execute().fetch_one()
+        result = self.find("_id = :id").bind("id", doc_id).execute()
+        doc = result.fetch_one()
+        self._connection.fetch_active_result()
+        return doc
 
     def remove_one(self, doc_id):
         """Removes a Document matching the Document ID.
@@ -483,7 +500,9 @@ class Table(DatabaseObject):
         Returns:
             mysqlx.SelectStatement: SelectStatement object
         """
-        return SelectStatement(self, *fields)
+        stmt = SelectStatement(self, *fields)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def insert(self, *fields):
         """Creates a new :class:`mysqlx.InsertStatement` object.
@@ -494,7 +513,9 @@ class Table(DatabaseObject):
         Returns:
             mysqlx.InsertStatement: InsertStatement object
         """
-        return InsertStatement(self, *fields)
+        stmt = InsertStatement(self, *fields)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def update(self):
         """Creates a new :class:`mysqlx.UpdateStatement` object.
@@ -502,7 +523,9 @@ class Table(DatabaseObject):
         Returns:
             mysqlx.UpdateStatement: UpdateStatement object
         """
-        return UpdateStatement(self)
+        stmt = UpdateStatement(self)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def delete(self):
         """Creates a new :class:`mysqlx.DeleteStatement` object.
@@ -513,7 +536,9 @@ class Table(DatabaseObject):
         .. versionchanged:: 8.0.12
            The ``condition`` parameter was removed.
         """
-        return DeleteStatement(self)
+        stmt = DeleteStatement(self)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def count(self):
         """Counts the rows in the table.
