@@ -1,4 +1,4 @@
-# Copyright (c) 2016, 2017, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -124,6 +124,7 @@ class TokenType(object):
     DAY_MINUTE = 86
     DAY_HOUR = 87
     YEAR_MONTH = 88
+    OVERLAPS = 89
 # pylint: enable=C0103
 
 _INTERVAL_UNITS = set([
@@ -157,6 +158,7 @@ _RESERVED_WORDS = {
     "not":      TokenType.NOT,
     "like":     TokenType.LIKE,
     "in":       TokenType.IN,
+    "overlaps": TokenType.OVERLAPS,
     "regexp":   TokenType.REGEXP,
     "between":  TokenType.BETWEEN,
     "interval": TokenType.INTERVAL,
@@ -235,6 +237,7 @@ _OPERATORS = {
     "<": "<",
     "<=": "<=",
     "&": "&",
+    "&&": "&&",
     "|": "|",
     "<<": "<<",
     ">>": ">>",
@@ -245,7 +248,8 @@ _OPERATORS = {
     "~": "~",
     "%": "%",
     "cast": "cast",
-    "cont_in": "cont_in"
+    "cont_in": "cont_in",
+    "overlaps": "overlaps"
 }
 
 _UNARY_OPERATORS = {
@@ -262,7 +266,8 @@ _NEGATION = {
     "regexp": "not_regexp",
     "like": "not_like",
     "in": "not_in",
-    "cont_in": "not_cont_in"
+    "cont_in": "not_cont_in",
+    "overlaps": "not_overlaps",
 }
 
 
@@ -290,18 +295,13 @@ def build_expr(value):
     if isinstance(value, (Message)):
         return value
     elif isinstance(value, (ExprParser)):
-        return value.expr()
+        return value.expr(reparse=True)
     elif isinstance(value, (dict, DbDoc)):
         msg["type"] = mysqlxpb_enum("Mysqlx.Expr.Expr.Type.OBJECT")
         msg["object"] = build_object(value).get_message()
     elif isinstance(value, (list, tuple)):
         msg["type"] = mysqlxpb_enum("Mysqlx.Expr.Expr.Type.ARRAY")
         msg["array"] = build_array(value).get_message()
-    # look for MySQL expressions
-    elif isinstance(value, STRING_TYPES) and \
-        (("(" in value and ")" in value) or ("{" in value and "}" in value)):
-        expr_parser = ExprParser(value, False)
-        return expr_parser.expr()
     else:
         msg["type"] = mysqlxpb_enum("Mysqlx.Expr.Expr.Type.LITERAL")
         msg["literal"] = build_scalar(value).get_message()
@@ -423,6 +423,9 @@ class ExprParser(object):
         self.clean_expression()
         self.lex()
 
+    def __str__(self):
+        return "<mysqlx.ExprParser '{}'>".format(self.string)
+
     def clean_expression(self):
         """Removes the keywords that does not form part of the expression.
 
@@ -551,7 +554,7 @@ class ExprParser(object):
                     token = Token(TokenType.EQ, "==", 1)
             elif char == "&":
                 if self.next_char_is(i, "&"):
-                    token = Token(TokenType.ANDAND, char, 2)
+                    token = Token(TokenType.ANDAND, "&&", 2)
                 else:
                     token = Token(TokenType.BITAND, char)
             elif char == "^":
@@ -655,10 +658,10 @@ class ExprParser(object):
         exprs = []
         self.consume_token(TokenType.LPAREN)
         if not self.cur_token_type_is(TokenType.RPAREN):
-            exprs.append(self.expr().get_message())
+            exprs.append(self._expr().get_message())
             while self.cur_token_type_is(TokenType.COMMA):
                 self.pos += 1
-                exprs.append(self.expr().get_message())
+                exprs.append(self._expr().get_message())
         self.consume_token(TokenType.RPAREN)
         return exprs
 
@@ -850,7 +853,7 @@ class ExprParser(object):
         msg = Message("Mysqlx.Expr.Array")
         while self.pos < len(self.tokens) and \
             not self.cur_token_type_is(TokenType.RSQBRACKET):
-            msg["value"].extend([self.expr().get_message()])
+            msg["value"].extend([self._expr().get_message()])
             if not self.cur_token_type_is(TokenType.COMMA):
                 break
             self.consume_token(TokenType.COMMA)
@@ -872,7 +875,7 @@ class ExprParser(object):
             item = Message("Mysqlx.Expr.Object.ObjectField")
             item["key"] = self.consume_token(TokenType.LSTRING)
             self.consume_token(TokenType.COLON)
-            item["value"] = self.expr().get_message()
+            item["value"] = self._expr().get_message()
             msg["fld"].extend([item.get_message()])
             if not self.cur_token_type_is(TokenType.COMMA):
                 break
@@ -896,7 +899,6 @@ class ExprParser(object):
             raise ValueError("Invalid placeholder name at token pos {0}"
                              "".format(self.pos))
 
-        place_holder_name = place_holder_name.lower()
         msg_expr = Message("Mysqlx.Expr.Expr")
         msg_expr["type"] = mysqlxpb_enum("Mysqlx.Expr.Expr.Type.PLACEHOLDER")
         if place_holder_name in self.placeholder_name_to_position:
@@ -914,7 +916,7 @@ class ExprParser(object):
         """
         operator = Message("Mysqlx.Expr.Operator", name="cast")
         self.consume_token(TokenType.LPAREN)
-        operator["param"].extend([self.expr().get_message()])
+        operator["param"].extend([self._expr().get_message()])
         self.consume_token(TokenType.AS)
 
         type_scalar = build_bytes_scalar(str.encode(self.cast_data_type()))
@@ -971,7 +973,7 @@ class ExprParser(object):
             dimension.append(self.consume_token(TokenType.LNUM))
         self.consume_token(TokenType.RPAREN)
 
-        return "({0})".format(dimension[0]) if len(dimension) is 1 else \
+        return "({0})".format(dimension[0]) if len(dimension) == 1 else \
                "({0},{1})".format(*dimension)
 
     def star_operator(self):
@@ -993,7 +995,7 @@ class ExprParser(object):
         elif token.token_type == TokenType.CAST:
             return self.cast()
         elif token.token_type == TokenType.LPAREN:
-            expr = self.expr()
+            expr = self._expr()
             self.expect_token(TokenType.RPAREN)
             return expr
         elif token.token_type in [TokenType.PLUS, TokenType.MINUS]:
@@ -1128,6 +1130,10 @@ class ExprParser(object):
                 else:
                     op_name = "cont_in"
                     params.append(self.comp_expr().get_message())
+            elif self.cur_token_type_is(TokenType.OVERLAPS):
+                self.consume_token(TokenType.OVERLAPS)
+                params.append(self.comp_expr().get_message())
+
             elif self.cur_token_type_is(TokenType.LIKE):
                 self.consume_token(TokenType.LIKE)
                 params.append(self.comp_expr().get_message())
@@ -1170,9 +1176,26 @@ class ExprParser(object):
         return self.parse_left_assoc_binary_op_expr(
             set([TokenType.OR, TokenType.OROR]), self.xor_expr)
 
-    def expr(self):
+    def _expr(self, reparse=False):
+        if reparse:
+            self.tokens = []
+            self.pos = 0
+            self.placeholder_name_to_position = {}
+            self.positional_placeholder_count = 0
+            self.lex()
         return self.or_expr()
 
+    def expr(self, reparse=False):
+        expression = self._expr(reparse)
+        used_tokens = self.pos
+        if self.pos_token_type_is(len(self.tokens) - 2, TokenType.AS):
+            used_tokens += 2
+        if used_tokens < len(self.tokens):
+            raise ValueError("Unused token types {} found in expression at "
+                             "position: {}".format(self.tokens[self.pos:],
+                                                   self.pos))
+        return expression
+
     def parse_table_insert_field(self):
         return Message("Mysqlx.Crud.Column",
                        name=self.consume_token(TokenType.IDENT))
@@ -1201,7 +1224,7 @@ class ExprParser(object):
             if not first:
                 self.consume_token(TokenType.COMMA)
             first = False
-            projection = Message("Mysqlx.Crud.Projection", source=self.expr())
+            projection = Message("Mysqlx.Crud.Projection", source=self._expr())
             if self.cur_token_type_is(TokenType.AS):
                 self.consume_token(TokenType.AS)
                 projection["alias"] = self.consume_token(TokenType.IDENT)
@@ -1218,7 +1241,7 @@ class ExprParser(object):
             if not first:
                 self.consume_token(TokenType.COMMA)
             first = False
-            order = Message("Mysqlx.Crud.Order", expr=self.expr())
+            order = Message("Mysqlx.Crud.Order", expr=self._expr())
             if self.cur_token_type_is(TokenType.ORDERBY_ASC):
                 order["direction"] = mysqlxpb_enum(
                     "Mysqlx.Crud.Order.Direction.ASC")
@@ -1237,5 +1260,5 @@ class ExprParser(object):
             if not first:
                 self.consume_token(TokenType.COMMA)
             first = False
-            expr_list.append(self.expr().get_message())
+            expr_list.append(self._expr().get_message())
         return expr_list
